import{s as a,r as t,H as i,j as d,b4 as c,b5 as e,D as l}from"./index.DvRPFfw6.js";const g=a("iframe",{target:"e3tg43n0"})(({theme:o,disableScrolling:r})=>({width:"100%",colorScheme:"normal",border:"none",padding:o.spacing.none,margin:o.spacing.none,overflow:r?"hidden":void 0}));function s(o){return l(o)||o===""?void 0:o}function I({element:o}){const r=s(o.src),n=i(r)?void 0:s(o.srcdoc);return d(g,{className:"stIFrame","data-testid":"stIFrame",allow:e,disableScrolling:!o.scrolling,src:r,srcDoc:n,height:o.height,scrolling:o.scrolling?"auto":"no",sandbox:c,title:"st.iframe",tabIndex:o.tabIndex??void 0})}const m=t.memo(I);export{m as default};
