import{r as l,b8 as n,j as e,b9 as u,br as c,b7 as d,ba as B,bb as b}from"./index.DvRPFfw6.js";function m(i){const{disabled:o,element:t,widgetMgr:r,fragmentId:s}=i;let a=n.SECONDARY;return t.type==="primary"?a=n.PRIMARY:t.type==="tertiary"&&(a=n.TERTIARY),e(b,{className:"stButton","data-testid":"stButton",children:e(u,{help:t.help,containerWidth:!0,children:e(c,{kind:a,size:d.SMALL,disabled:o,containerWidth:!0,onClick:()=>r.setTriggerValue(t,{fromUi:!0},s),children:e(B,{icon:t.icon,label:t.label})})})})}const f=l.memo(m);export{f as default};
